@@ -10,14 +10,12 @@
 
 #[macro_use]
 extern crate lazy_static;
-extern crate clipboard;
+extern crate copypasta;
 extern crate fontdue;
 
 #[cfg(not(target_arch = "wasm32"))]
 extern crate sysinfo;
 
-use message::PopupMessage;
-use popup::Placement;
 pub use rg3d_core as core;
 
 pub mod border;
@@ -26,6 +24,7 @@ pub mod button;
 pub mod canvas;
 pub mod check_box;
 pub mod color;
+pub mod curve;
 pub mod decorator;
 pub mod dock;
 pub mod draw;
@@ -73,13 +72,15 @@ use crate::{
     draw::{CommandTexture, Draw, DrawingContext},
     message::{
         ButtonState, CursorIcon, KeyboardModifiers, MessageData, MessageDirection, MouseButton,
-        OsEvent, UiMessage, UiMessageData, WidgetMessage,
+        OsEvent, PopupMessage, UiMessage, UiMessageData, WidgetMessage,
     },
     node::UINode,
+    popup::Placement,
     ttf::{Font, SharedFont},
     widget::{Widget, WidgetBuilder},
 };
-use clipboard::{ClipboardContext, ClipboardProvider};
+
+use copypasta::ClipboardContext;
 use std::{
     cell::Cell,
     collections::{HashMap, HashSet, VecDeque},
@@ -476,7 +477,7 @@ pub struct UserInterface<M: MessageData, C: Control<M, C>> {
 lazy_static! {
     pub static ref DEFAULT_FONT: SharedFont = {
         let font_bytes = std::include_bytes!("./built_in_font.ttf").to_vec();
-        let font = Font::from_memory(font_bytes, 20.0, Font::default_char_set()).unwrap();
+        let font = Font::from_memory(font_bytes, 16.0, Font::default_char_set()).unwrap();
         Arc::new(Mutex::new(font)).into()
     };
 }
