@@ -13,17 +13,18 @@
 //! However [WidgetMessage::GotFocus](enum.WidgetMessage.html) has "Direction: From UI" which means that only
 //! internal library code can send such messages without a risk of breaking anything.
 
-use crate::core::algebra::{Vector2, Vector3};
-use crate::dock::SplitDirection;
-use crate::formatted_text::WrapMode;
 use crate::{
     brush::Brush,
     core::{
+        algebra::{Vector2, Vector3},
         color::{Color, Hsv},
+        curve::{Curve, CurveKeyKind},
         pool::Handle,
     },
-    dock::TileContent,
+    dock::{SplitDirection, TileContent},
     draw::SharedTexture,
+    file_browser::Filter,
+    formatted_text::WrapMode,
     messagebox::MessageBoxResult,
     popup::Placement,
     ttf::SharedFont,
@@ -564,6 +565,7 @@ pub enum FileSelectorMessage {
     Path(PathBuf),
     Commit(PathBuf),
     Cancel,
+    Filter(Option<Filter>),
 }
 
 impl FileSelectorMessage {
@@ -571,6 +573,7 @@ impl FileSelectorMessage {
     define_constructor_unbound!(FileSelector(FileSelectorMessage:Root) => fn root(Option<PathBuf>), layout: false);
     define_constructor_unbound!(FileSelector(FileSelectorMessage:Path) => fn path(PathBuf), layout: false);
     define_constructor_unbound!(FileSelector(FileSelectorMessage:Cancel) => fn cancel(), layout: false);
+    define_constructor_unbound!(FileSelector(FileSelectorMessage:Filter) => fn filter(Option<Filter>), layout: false);
 }
 
 #[derive(Copy, Clone, PartialOrd, PartialEq, Ord, Eq, Hash, Debug)]
@@ -644,11 +647,13 @@ impl<M: MessageData, C: Control<M, C>> TreeRootMessage<M, C> {
 pub enum FileBrowserMessage {
     Root(Option<PathBuf>),
     Path(PathBuf),
+    Filter(Option<Filter>),
 }
 
 impl FileBrowserMessage {
     define_constructor_unbound!(FileBrowser(FileBrowserMessage:Root) => fn root(Option<PathBuf>), layout: false);
     define_constructor_unbound!(FileBrowser(FileBrowserMessage:Path) => fn path(PathBuf), layout: false);
+    define_constructor_unbound!(FileBrowser(FileBrowserMessage:Filter) => fn filter(Option<Filter>), layout: false);
 }
 
 #[derive(Debug, Clone, PartialEq)]
@@ -894,6 +899,33 @@ impl ColorFieldMessage {
     define_constructor_unbound!(ColorField(ColorFieldMessage:Color) => fn color(Color), layout: false);
 }
 
+#[derive(Debug, Clone, PartialEq)]
+pub enum CurveEditorMessage {
+    Sync(Curve),
+    ViewPosition(Vector2<f32>),
+    Zoom(f32),
+    ZoomToFit,
+
+    // Internal messages. Use only when you know what you're doing.
+    // These are internal because you must use Sync message to request changes
+    // in the curve editor.
+    ChangeSelectedKeysKind(CurveKeyKind),
+    RemoveSelection,
+    // Position in screen coordinates.
+    AddKey(Vector2<f32>),
+}
+
+impl CurveEditorMessage {
+    define_constructor_unbound!(CurveEditor(CurveEditorMessage:Sync) => fn sync(Curve), layout: false);
+    define_constructor_unbound!(CurveEditor(CurveEditorMessage:ViewPosition) => fn view_position(Vector2<f32>), layout: false);
+    define_constructor_unbound!(CurveEditor(CurveEditorMessage:Zoom) => fn zoom(f32), layout: false);
+    define_constructor_unbound!(CurveEditor(CurveEditorMessage:ZoomToFit) => fn zoom_to_fit(), layout: false);
+    // Internal. Use only when you know what you're doing.
+    define_constructor_unbound!(CurveEditor(CurveEditorMessage:RemoveSelection) => fn remove_selection(), layout: false);
+    define_constructor_unbound!(CurveEditor(CurveEditorMessage:ChangeSelectedKeysKind) => fn change_selected_keys_kind(CurveKeyKind), layout: false);
+    define_constructor_unbound!(CurveEditor(CurveEditorMessage:AddKey) => fn add_key(Vector2<f32>), layout: false);
+}
+
 #[derive(Debug, Clone, PartialEq)]
 pub enum UiMessageData<M: MessageData, C: Control<M, C>> {
     Widget(WidgetMessage<M, C>),
@@ -927,6 +959,7 @@ pub enum UiMessageData<M: MessageData, C: Control<M, C>> {
     ColorField(ColorFieldMessage),
     Expander(ExpanderMessage),
     SaturationBrightnessField(SaturationBrightnessFieldMessage),
+    CurveEditor(CurveEditorMessage),
     User(M),
 }
 
